"{ Package: 'stx:libjava' }"

JavaConstantPoolContent subclass:#JavaNameAndType2
	instanceVariableNames:'nameIndex descriptorIndex name descriptor'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Java-Reader-Support-new'
!

!JavaNameAndType2 class methodsFor:'documentation'!

documentation
"
    I represent NameAndTypeInfo structure found in java constant pool. 

    [author:]
        Marcel Hlopko <hlopkmar@fel.cvut.cz>

    [instance variables:]
        name - string found in constant pool at nameIndex. Represents field or method name.
        descriptor - string found in constant pool at descIndex. Represents field or method type.

    [class variables:]

    [see also:]

"
! !

!JavaNameAndType2 class methodsFor:'instance creation'!

in: aJavaConstantPool withNameAt: arg1 andDescriptorAt: arg2 
    "Create & return a new instance for arg."
    
    ^ self basicNew 
        initializeIn: aJavaConstantPool
        withNameAt: arg1
        andDescriptorAt: arg2

    "Created: / 10-05-2011 / 15:45:28 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'accessing'!

descriptor
    descriptor ifNil: [self resolve].
    ^ descriptor.

    "Created: / 08-04-2011 / 11:55:33 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 15:46:14 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

name
    name ifNil: [self resolve].
    ^ name.

    "Created: / 08-04-2011 / 11:55:23 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 15:49:59 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

selector
    self isResolved ifFalse: [ self resolve ].
    ^ (name , descriptor) asSymbol.

    "Created: / 11-04-2011 / 21:31:27 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 17:29:47 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'comparing'!

= aNameAndType 
    name ~= aNameAndType name ifTrue: [ ^ false ].
    ^ descriptor = aNameAndType descriptor.

    "Created: / 08-04-2011 / 11:56:35 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

hash
    ^ name hash bitXor: descriptor hash.

    "Created: / 08-04-2011 / 11:57:35 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'initialization'!

initializeIn: aJavaConstantPool withNameAt: nameCPIndex andDescriptorAt: descriptorCPIndex 
    nameIndex := nameCPIndex.
    descriptorIndex := descriptorCPIndex.
    constantPool := aJavaConstantPool.
    super initialize.

    "Created: / 10-05-2011 / 15:45:00 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'printing'!

printString
^ 'JavaNameAndType name=' , name printString , ' descriptor= ' , descriptor printString.

    "Created: / 10-05-2011 / 14:17:38 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'queries'!

isJavaNameAndType
    "return true, if given object represents name and type struct in java constant pool"

    ^ true.

    "Created: / 10-05-2011 / 12:23:45 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

isNewJavaNameAndType
    "only temporary, to know if name and type is old (claus' version) or new (m version)"
    
    ^ true.

    "Created: / 10-05-2011 / 12:24:16 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

isResolved
    name ifNil: [^ false].
    descriptor ifNil: [^ false].
    ^ true.

    "Created: / 10-05-2011 / 17:27:39 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 methodsFor:'resolving'!

findResolvedStaticValue
    self halt: 'should not need - ask mh'.

    "Created: / 10-05-2011 / 17:26:59 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

findResolvedValue
    name := constantPool at: nameIndex.
    descriptor := constantPool at: descriptorIndex.
    name ifNil: [ self warning: 'name nil in ' , self printString ].
    descriptor 
            ifNil: [ self warning: 'descriptor nil in ' , self printString ].

    "Created: / 10-05-2011 / 17:27:08 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

invalidate
    name := nil.
    descriptor := nil.

    "Created: / 10-05-2011 / 17:28:34 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

invalidateForClass: arg 
    self halt: 'should not need - tell mh'.

    "Created: / 10-05-2011 / 17:28:53 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

resolve
    (name isNil or: [descriptor isNil]) ifTrue: [self findResolvedValue].

    "Created: / 10-05-2011 / 15:46:14 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 17:28:19 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

resolveStatic
   ^ self resolve.

    "Created: / 10-05-2011 / 17:25:03 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaNameAndType2 class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
